@@ -3,31 +3,37 @@
 ** Copyright (C) 2011-2012 Denis Shienkov <denis.shienkov@gmail.com>
 ** Copyright (C) 2011 Sergey Belyashov <Sergey.Belyashov@gmail.com>
 ** Copyright (C) 2012 Laszlo Papp <lpapp@kde.org>
-** Contact: http://www.qt.io/licensing/
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the QtSerialPort module of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL21$
+** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
 ** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see http://www.qt.io/terms-conditions. For further
-** information use the contact form at http://www.qt.io/contact-us.
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file. Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** General Public License version 3 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL3 included in the
+** packaging of this file. Please review the following information to
+** ensure the GNU Lesser General Public License version 3 requirements
+** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
 **
-** As a special exception, The Qt Company gives you certain additional
-** rights. These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 2.0 or (at your option) the GNU General
+** Public license version 3 or any later version approved by the KDE Free
+** Qt Foundation. The licenses are as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-2.0.html and
+** https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -52,10 +58,7 @@
 #include <private/qringbuffer_p.h>
 #include <private/qiodevice_p.h>
 
-#if defined(Q_OS_WINCE)
-#  include <QtCore/qmutex.h>
-#  include <qt_windows.h>
-#elif defined(Q_OS_WIN32)
+#if defined(Q_OS_WIN32)
 #  include <qt_windows.h>
 #elif defined(Q_OS_UNIX)
 #  include <QtCore/qlockfile.h>
@@ -109,12 +112,8 @@ QString serialPortLockFilePath(const QString &portName);
 class QSerialPortErrorInfo
 {
 public:
-    explicit QSerialPortErrorInfo(QSerialPort::SerialPortError errorCode = QSerialPort::UnknownError,
-                                  const QString &errorString = QString())
-        : errorCode(errorCode)
-        , errorString(errorString)
-    {
-    }
+    explicit QSerialPortErrorInfo(QSerialPort::SerialPortError newErrorCode = QSerialPort::UnknownError,
+                                  const QString &newErrorString = QString());
     QSerialPort::SerialPortError errorCode;
     QString errorString;
 };
@@ -130,8 +129,6 @@ public:
 
     QSerialPortPrivate();
 
-    static int timeoutValue(int msecs, int elapsed);
-
     bool open(QIODevice::OpenMode mode);
     void close();
 
@@ -155,7 +152,6 @@ public:
     bool setParity(QSerialPort::Parity parity);
     bool setStopBits(QSerialPort::StopBits stopBits);
     bool setFlowControl(QSerialPort::FlowControl flowControl);
-    bool setDataErrorPolicy(QSerialPort::DataErrorPolicy policy);
 
     QSerialPortErrorInfo getSystemError(int systemErrorCode = -1) const;
 
@@ -166,8 +162,9 @@ public:
     static QString portNameToSystemLocation(const QString &port);
     static QString portNameFromSystemLocation(const QString &location);
 
-    static qint32 baudRateFromSetting(qint32 setting);
+#if defined(Q_OS_UNIX)
     static qint32 settingFromBaudRate(qint32 baudRate);
+#endif
 
     static QList<qint32> standardBaudRates();
 
@@ -181,42 +178,18 @@ public:
     QSerialPort::Parity parity;
     QSerialPort::StopBits stopBits;
     QSerialPort::FlowControl flowControl;
-    QSerialPort::DataErrorPolicy policy;
     bool settingsRestoredOnClose;
     bool isBreakEnabled;
 
-#if defined(Q_OS_WINCE)
-
-    bool initialize(DWORD eventMask);
-    bool updateDcb();
-    bool updateCommTimeouts();
-
-    bool waitForReadOrWrite(bool *selectForRead, bool *selectForWrite,
-                            bool checkRead, bool checkWrite,
-                            int msecs);
-    void processIoErrors(bool error);
-
-    bool notifyRead();
-    bool notifyWrite();
-
-    DCB currentDcb;
-    DCB restoredDcb;
-    COMMTIMEOUTS currentCommTimeouts;
-    COMMTIMEOUTS restoredCommTimeouts;
-    HANDLE handle;
-    bool parityErrorOccurred;
-
-    QThread *eventNotifier;
-    QMutex settingsChangeMutex;
-
-#elif defined(Q_OS_WIN32)
+#if defined(Q_OS_WIN32)
 
     bool initialize();
-    bool updateDcb();
-    bool updateCommTimeouts();
-    void handleLineStatusErrors();
+    bool setDcb(DCB *dcb);
+    bool getDcb(DCB *dcb);
     OVERLAPPED *waitForNotified(int msecs);
 
+    qint64 queuedBytesCount(QSerialPort::Direction direction) const;
+
     bool completeAsyncCommunication(qint64 bytesTransferred);
     bool completeAsyncRead(qint64 bytesTransferred);
     bool completeAsyncWrite(qint64 bytesTransferred);
@@ -226,16 +199,15 @@ public:
     bool _q_startAsyncWrite();
     void _q_notified(DWORD numberOfBytes, DWORD errorCode, OVERLAPPED *overlapped);
 
-    bool emulateErrorPolicy();
     void emitReadyRead();
 
-    DCB currentDcb;
     DCB restoredDcb;
     COMMTIMEOUTS currentCommTimeouts;
     COMMTIMEOUTS restoredCommTimeouts;
     HANDLE handle;
-    bool parityErrorOccurred;
     QByteArray readChunkBuffer;
+    QByteArray writeChunkBuffer;
+    bool communicationStarted;
     bool writeStarted;
     bool readStarted;
     QWinOverlappedIoNotifier *notifier;
@@ -245,19 +217,15 @@ public:
     OVERLAPPED writeCompletionOverlapped;
     DWORD originalEventMask;
     DWORD triggeredEventMask;
-    qint64 actualBytesToWrite;
 
 #elif defined(Q_OS_UNIX)
 
     bool initialize(QIODevice::OpenMode mode);
-    bool updateTermios();
+    bool setTermios(const termios *tio);
+    bool getTermios(termios *tio);
 
-    QSerialPortErrorInfo setBaudRate_helper(qint32 baudRate,
-            QSerialPort::Directions directions);
-    QSerialPortErrorInfo setCustomBaudRate(qint32 baudRate,
-            QSerialPort::Directions directions);
-    QSerialPortErrorInfo setStandardBaudRate(qint32 baudRate,
-            QSerialPort::Directions directions);
+    bool setCustomBaudRate(qint32 baudRate, QSerialPort::Directions directions);
+    bool setStandardBaudRate(qint32 baudRate, QSerialPort::Directions directions);
 
     bool isReadNotificationEnabled() const;
     void setReadNotificationEnabled(bool enable);
@@ -274,13 +242,11 @@ public:
 #ifndef CMSPAR
     qint64 writePerChar(const char *data, qint64 maxSize);
 #endif
-    qint64 readPerChar(char *data, qint64 maxSize);
 
     bool readNotification();
     bool startAsyncWrite();
     bool completeAsyncWrite();
 
-    struct termios currentTermios;
     struct termios restoredTermios;
     int descriptor;
 
