@@ -3,31 +3,37 @@
 ** Copyright (C) 2011-2012 Denis Shienkov <denis.shienkov@gmail.com>
 ** Copyright (C) 2011 Sergey Belyashov <Sergey.Belyashov@gmail.com>
 ** Copyright (C) 2012 Laszlo Papp <lpapp@kde.org>
-** Contact: http://www.qt.io/licensing/
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the QtSerialPort module of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL21$
+** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
 ** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see http://www.qt.io/terms-conditions. For further
-** information use the contact form at http://www.qt.io/contact-us.
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file. Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** General Public License version 3 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL3 included in the
+** packaging of this file. Please review the following information to
+** ensure the GNU Lesser General Public License version 3 requirements
+** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
 **
-** As a special exception, The Qt Company gives you certain additional
-** rights. These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 2.0 or (at your option) the GNU General
+** Public license version 3 or any later version approved by the KDE Free
+** Qt Foundation. The licenses are as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-2.0.html and
+** https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -36,6 +42,7 @@
 #include "qserialportinfo.h"
 #include "qserialportinfo_p.h"
 #include "qserialport.h"
+#include "qserialport_p.h"
 
 QT_BEGIN_NAMESPACE
 
@@ -72,7 +79,7 @@ QSerialPortInfo::QSerialPortInfo()
     Constructs a copy of \a other.
 */
 QSerialPortInfo::QSerialPortInfo(const QSerialPortInfo &other)
-    : d_ptr(other.d_ptr ? new QSerialPortInfoPrivate(*other.d_ptr) : Q_NULLPTR)
+    : d_ptr(other.d_ptr ? new QSerialPortInfoPrivate(*other.d_ptr) : nullptr)
 {
 }
 
@@ -80,13 +87,8 @@ QSerialPortInfo::QSerialPortInfo(const QSerialPortInfo &other)
     Constructs a QSerialPortInfo object from serial \a port.
 */
 QSerialPortInfo::QSerialPortInfo(const QSerialPort &port)
+    : QSerialPortInfo(port.portName())
 {
-    foreach (const QSerialPortInfo &serialPortInfo, availablePorts()) {
-        if (port.portName() == serialPortInfo.portName()) {
-            *this = serialPortInfo;
-            break;
-        }
-    }
 }
 
 /*!
@@ -98,9 +100,10 @@ QSerialPortInfo::QSerialPortInfo(const QSerialPort &port)
 */
 QSerialPortInfo::QSerialPortInfo(const QString &name)
 {
-    foreach (const QSerialPortInfo &serialPortInfo, availablePorts()) {
-        if (name == serialPortInfo.portName()) {
-            *this = serialPortInfo;
+    const auto infos = QSerialPortInfo::availablePorts();
+    for (const QSerialPortInfo &info : infos) {
+        if (name == info.portName()) {
+            *this = info;
             break;
         }
     }
@@ -257,6 +260,7 @@ bool QSerialPortInfo::hasProductIdentifier() const
     \sa isBusy()
 */
 
+#if QT_DEPRECATED_SINCE(5, 6)
 /*!
     \fn bool QSerialPortInfo::isBusy() const
 
@@ -265,6 +269,7 @@ bool QSerialPortInfo::hasProductIdentifier() const
 
     \sa isNull()
 */
+#endif // QT_DEPRECATED_SINCE(5, 6)
 
 #if QT_DEPRECATED_SINCE(5, 2)
 /*!
@@ -281,9 +286,13 @@ bool QSerialPortInfo::hasProductIdentifier() const
 /*!
     \fn QList<qint32> QSerialPortInfo::standardBaudRates()
 
-    Returns a list of available standard baud rates supported by
-    the current serial port.
+    Returns a list of available standard baud rates supported
+    by the target platform.
 */
+QList<qint32> QSerialPortInfo::standardBaudRates()
+{
+    return QSerialPortPrivate::standardBaudRates();
+}
 
 /*!
     \fn QList<QSerialPortInfo> QSerialPortInfo::availablePorts()
